(****************************************************)
(*           FAT16 Test program with MMC            *)
(* This is a small working sample which uses a MMC  *)
(* card an interface for storage.                   *)
(* If the desired file does not exist, it will be   *)
(* created and filled with some strings             *)
(* Then, regardless of the existance of the file    *)
(* a few string are appended to this file           *)
(****************************************************)

program FAT16MMC;

{ $BOOTRST $0F000}         {Reset Jump to $0F000}
{$NOSHADOW}
{ $W+ Warnings}            {Warnings off}

Device = mega128, VCC = 5;

Import SysTick, LCDmultiPort, TWImaster, BeepPort, FAT16;

From System Import LongWord, LongInt;

Define
  ProcClock      = 16000000;        {Hertz}
  SysTick        = 5;               {msec}
  StackSize      = $0050, iData;
  FrameSize      = $0100, iData;
  TWIpresc       = 10;
  LCDmultiPort   = I2C_TWI;
  LCDTYPE_M      = 44780;          // 66712;
  LCDrows_M      = 4;              {4 rows}
  LCDcolumns_M   = 16;             {16 chars per line}
  FAT16          = MMC_SPI, IData;
  F16_FileHandles= 2;
  F16_DirLevels  = 2;
  BeepPort       = PortG, 4;

Uses UFAT16;

Implementation

{$IDATA}
structconst
  kString : string = '27.03.03,08:48:19,48.70016,9.200158,449.4,1,4.7,04,0.8,001,X,XL,log,,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0'+#13+#10;

var
  S              : tPathStr;
  Fn             : tFileName;
  SR             : TSearchRec;
  F              : File;
  Res, ww, x     : Word;
  bool           : boolean;
  st             : string[10];
  ptr            : pointer;


// functions
function WriteStrToFile : boolean;                 // an append function
var ptr : pointer;
begin
  mDelay(200);
  LCDxy_M(LCD_m1, 0, 2);
  LCDCLRLINE_M(LCD_m1, 2);
  if not F16_FileAssign(F, '', 'Test.txt') then    // always the first step
    Write(LCDOut_M,  'open error');                // to crete a file handle
    return(false);
  else
    Write(LCDOut_M,  'open file');
  endif;
  mDelay(200);
  LCDCLRLINE_M(LCD_m1, 2);
  if not F16_FileAppend(F) then                    // we open the file for
    Write(LCDOut_M,  'append error');              // append operations
    F16_FileClose(F);
    return(false);
  else
    Write(LCDOut_M,  'append');
  endif;
  mDelay(200);
  LCDCLRLINE_M(LCD_m1, 2);
  ww:= word(length(kString));                       // byte counter
  ptr:= @kString;                                   // point to string
  inc(ptr);                                         // point to first char in string
  F16_BlockWrite(F, ptr, ww, Res);
  Write(LCDOut_M,  'close file');
  F16_FileClose(F);
  mDelay(200);
  return(true);
end;

{ Main Program }
begin
  EnableInts;
  LCDsetup_M(LCD_m1);
  LCDcursor_M(LCD_m1, true, false);
  Write(LCDOut_M,  'E-LAB MMC Test');
  BeepOut(2500, 20);
  mDelay(300);

  bool:= F16_DiskInit;                              // always the very fist step
  LCDxy_M(LCD_m1, 0, 1);                            // the hardware is initialized
  if bool then
    Write(LCDOut_M, 'MMC init ok!');
    BeepOut(1000, 5);
  else
    Write(LCDOut_M, 'MMC init fail!');
    BeepSiren(0, 3);
    loop
      nop;
    endloop;
  endif;
  mDelay(300);

  bool:= F16_DiskReset;                             // we reset the disk and the
  LCDxy_M(LCD_m1, 0, 2);                            // driver to its default state
  if bool then                                      // and read some disk parameters
    Write(LCDOut_M, 'MMC reset ok!');
    BeepOut(1000, 5);
  else
    Write(LCDOut_M, 'MMC reset fail!');
    BeepSiren(0, 3);
    loop
      nop;
    endloop;
  endif;
  mDelay(300);

  LCDxy_M(LCD_m1, 0, 3);
  if F16_CheckDisk then                             // we check for an existing FAT16 system
    Write(LCDOut_M, 'Size '+LongToStr(F16_GetDiskSize));
    BeepOut(1000, 5);
  else
    Write(LCDOut_M, 'CheckDisk fail!');
    BeepSiren(0, 3);
    loop
      nop;
    endloop;
  endif;
  mDelay(300);
  
  // if this file does not exist we must create it and fill it with initial data
  if not F16_FileExist('', 'Test.txt', [faAnyFile]) then
    if not F16_FileAssign(F, '', 'Test.txt') then
      LCDxy_M(LCD_m1, 0, 1);
      LCDclrEOL_M(LCD_m1);
      Write(LCDOut_M,  'assign error');
      BeepSiren(0, 3);
      loop
        nop;
      endloop;
    endif;
    BeepOut(1000, 5);
    mDelay(300);

    if not F16_FileRewrite(F, [], 0, 0) then
      LCDxy_M(LCD_m1, 0, 3);
      LCDclrEOL_M(LCD_m1);
      F16_FileClose(F);
      Write(LCDOut_M,  'rewrite error');
      BeepSiren(0, 3);
      loop
        nop;
      endloop;
    endif;
    BeepOut(1000, 5);
    mDelay(300);

    LCDxy_M(LCD_m1, 0, 2);
    LCDclrEOL_M(LCD_m1);
    Write(LCDOut_M,  'writing file');
    while (x <= 400) do
      // write a data set to file
      ww:= word(length(kString));                       // byte counter
      ptr:= @kString;                                   // point to string
      inc(ptr);                                         // point to first char in string
      if not F16_BlockWrite(F, ptr, ww, Res) then
        BeepSiren(0, 3);
        LCDxy_M(LCD_m1, 0, 2);
        LCDclrEOL_M(LCD_m1);
        Write(LCDOut_M,  'write failed');
        F16_FileClose(F);
        loop
          nop;
        endloop;
      endif;
      LCDxy_M(LCD_m1, 0, 3);
      LCDclrEOL_M(LCD_m1);
      st:= IntToStr(x);
      Write(LCDOut_M, st);
      BeepOut(1000, 1);
      inc(x);
    endwhile;
    F16_FileClose(F);
    LCDxy_M(LCD_m1, 0, 3);
    LCDclrEOL_M(LCD_m1);
    Write(LCDOut_M,  'write finished');
    mDelay(2000);
  endif;

  // at this point the file already exists and we append some strings
  x:= 0;
  while x <= 20 do
    // append a data set to file
    if not WriteStrToFile then
      loop
        nop;
      endloop;
    endif;
    LCDxy_M(LCD_m1, 0, 3);
    LCDclrEOL_M(LCD_m1);
    st:= 'set '+IntToStr(x);
    Write(LCDOut_M, st);
    BeepOut(1000, 1);
    inc(x);
  endwhile;
  LCDxy_M(LCD_m1, 0, 3);
  LCDclrEOL_M(LCD_m1);
  Write(LCDOut_M,  'append finished');

  loop
    nop;
    nop;
  endloop;
end FAT16MMC.


